/* Blinken_XMMS : for XMMS - Cross-platform multimedia player
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License forore details.
*
*  You should have received a copy of the GNU General Public License
*  along with XMMS.
*/

/* Compile:
*  gcc -Wall --shared `xmms-config --cflags` -o Blinken_XMMS.so Blinken_XMMS.c
* Install:
*  install Blinken_XMMS.so `xmms-config --visualization-plugin-dir`
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <xmms/plugin.h>
#include <xmms/configfile.h>

#include "./configure.c"

#define NUM_BANDS        18
#define WIDTH            18
#define HEIGHT            8
#define FRAMEDURATION 40000

#define MSG  {0xFE,0xED,0xBE,0xEF,0x00,0x00,0x00,0x00,0x00,0x012,0x00,0x08,\
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,\
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,\
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,\
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,\
0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00,\
0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00,\
0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00,\
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}

static int sock;
static unsigned char msg[156]=MSG;
static unsigned char scalmsg[10] = {0x11, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0xAA, 0xCC, 0x00};

BlinkenXmmsConfig blinken_cfg;

static gdouble scale;
static gint16 bar_heights[NUM_BANDS];

void blinken_init();
void blinken_playback_start();
void blinken_playback_stop();
void blinken_render_freq(gint16 data[1][256]);

static long int timer;
struct timeval tp;

VisPlugin g_blinken_plugin =
{
        NULL,                   /* handle */
        NULL,                   /* filename */
        0,                      /* session id */
        "Blinken_XMMS",         /* description */
        0,                      /* PCM Channels */
        1,                      /* Freq. Channels */
        /* Callbacks ...    */
        blinken_init,           /* init */
        blinken_cleanup,        /* cleanup */
        NULL,                   /* about-dialog */
	blinken_configure,      /* configure-dialog */
        NULL,                   /* disable-plugin */
        blinken_playback_start, /* playback-start */
        blinken_playback_stop,  /* playback-top */
        NULL,                   /* render-pcm */
        blinken_render_freq     /* render-freq */
};


VisPlugin *get_vplugin_info(void)
{
        return( & g_blinken_plugin);
}

/* Read current settings from the xmms config file */
void blinken_read_config(void)
{
	ConfigFile *cfg;	
	gchar *filename;
	
	filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
	cfg = xmms_cfg_open_file(filename);
	
	if (cfg)
	{
		xmms_cfg_read_string(cfg, "Blinken_XMMS", "blinkenip", &blinken_cfg.ip);
		xmms_cfg_read_int(cfg, "Blinken_XMMS", "blinkenport", &blinken_cfg.port);
			
		xmms_cfg_free(cfg);
	}
	g_free(filename);
	
	if (blinken_cfg.ip==NULL) {
		blinken_cfg.ip = g_strdup("127.0.0.1");
		blinken_cfg.port = 2323;
	}
}

/* Save current settings to the xmms config file */
void blinken_save_config(void)
{
	ConfigFile *cfgfile;
	gchar *filename;

	filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
	cfgfile = xmms_cfg_open_file(filename);
	if (!cfgfile)
		cfgfile = xmms_cfg_new();
	xmms_cfg_write_string(cfgfile, "Blinken_XMMS", "blinkenip", blinken_cfg.ip);
	xmms_cfg_write_int(cfgfile, "Blinken_XMMS", "blinkenport", blinken_cfg.port);
	
	xmms_cfg_write_file(cfgfile, filename);
	xmms_cfg_free(cfgfile);
	g_free(filename);
}

void blinken_udp_init(void)
{
        if ((sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1)
        {
                printf("couldn't create socket");
                blinken_cleanup();
                exit(1);
        }
	
        struct sockaddr_in dest_addr;
        memset(&dest_addr, 0, sizeof(dest_addr));
        dest_addr.sin_family      = AF_INET;
	dest_addr.sin_port        = htons(blinken_cfg.port);
	dest_addr.sin_addr.s_addr = inet_addr(blinken_cfg.ip);

        if (connect(sock, (struct sockaddr*) &dest_addr, sizeof(dest_addr)) == -1)
        {
		printf("couldn't connect to %s:%d\n",blinken_cfg.ip,blinken_cfg.port);
		blinken_configure();
                blinken_cleanup();
                exit(2);
        }

	//printf("connected to %s:%d\n",blinken_cfg.ip,blinken_cfg.port);
        send(sock, msg, 156, 0);
}

void blinken_init()
{
	
	if (blinken_cfg.ip==NULL)
		blinken_read_config();
	blinken_udp_init();
	scale = (HEIGHT + 1)/ log(256);
}

void blinken_cleanup()
{
        close(sock);
}


void blinken_playback_start()
{
	gettimeofday(&tp,NULL);
	timer=tp.tv_usec;
	//printf( "playback_start\n");
}


void blinken_playback_stop()
{
        unsigned char msg[156]=MSG;
        send(sock, msg, 156, 0);
}


void blinken_render_freq( gint16 data[1][256])
{
	gettimeofday(&tp,NULL);
	if ((tp.tv_usec-timer)<FRAMEDURATION && (tp.tv_usec-timer)>0)
		return;
	//printf( "playback_run %ld\n",tp.tv_usec-timer);
	timer=tp.tv_usec;
        int result,zeile,spalte;
        int i,j,c;
        gint y;
        gint xscale[NUM_BANDS] = {0, 1, 2, 3, 5, 7, 10, 14, 20, 28, 40, 54, 74, 85, 101, 137, 140, 150};

        for(i = 0; i < NUM_BANDS; i++)
        {
                for(c = xscale[i], y = 0; c < xscale[i + 1]; c++)
                {
                        if(data[0][c] > y)
                        {
                                y = data[0][c];
                        }

                }

                y >>= 7;
                if(y != 0)
                {
                        y = (gint)(log(y) * scale);
                        if(y > HEIGHT+1)
                        {
                                y = HEIGHT+1;
                        }


                }

                bar_heights[i] = -y+9;

        }

        spalte = 0;
        zeile  = 0;
        for(i=12;i<=155;i++)
        {
                j = i-12;
                zeile  = j/18;
                spalte = j%18;

                if((bar_heights[spalte])==zeile+1)
                {
                        msg[i]=0xFF;
                }
                else if((bar_heights[spalte])<zeile+1)
                {
                        msg[i]=scalmsg[bar_heights[spalte]];
                }
                else
                {
                        msg[i]=0x00;
                }

        }

        result = send(sock, msg, 156, 0);
        return;
}
